#pragma once

#include "mcrcommon.h"

#pragma pack(push, 4)


#define INVALID_DST_ENCODED           0x00000001UL
#define INVALID_FREEWHEEL_DISPLAY     0x00000002UL
#define INVALID_FREEWHEEL_SYNCHRONIZE 0x00000004UL
#define INVALID_TIME_CODE_LOSS_NOTIFY 0x00000008UL
#define INVALID_TIME_CODE_SOURCE      0x00000010UL
#define INVALID_TIME_ZONE_BIAS        0x00000020UL
#define INVALID_STANDARD_BIAS         0x00000040UL
#define INVALID_STANDARD_YEAR         0x00000080UL
#define INVALID_STANDARD_MONTH        0x00000100UL
#define INVALID_STANDARD_DAY_OF_WEEK  0x00000200UL
#define INVALID_STANDARD_DATE         0x00000400UL
#define INVALID_STANDARD_HOUR         0x00000800UL
#define INVALID_STANDARD_MINUTE       0x00001000UL
#define INVALID_DAYLIGHT_BIAS         0x00002000UL
#define INVALID_DAYLIGHT_YEAR         0x00004000UL
#define INVALID_DAYLIGHT_MONTH        0x00008000UL
#define INVALID_DAYLIGHT_DAY_OF_WEEK  0x00010000UL
#define INVALID_DAYLIGHT_DATE         0x00020000UL
#define INVALID_DAYLIGHT_HOUR         0x00040000UL
#define INVALID_DAYLIGHT_MINUTE       0x00080000UL
#define INVALID_NMEA_CONFIGURATION    0x00100000UL

// bitmap for enabled NMEA messages
#define NMEA_MESSAGE_MASK		0x7F
#define NMEA_MESSAGE_BIT_ZDA    0x01
#define NMEA_MESSAGE_BIT_GSV    0x02
#define NMEA_MESSAGE_BIT_GGA    0x04
#define NMEA_MESSAGE_BIT_GLL    0x08
#define NMEA_MESSAGE_BIT_GSA    0x10
#define NMEA_MESSAGE_BIT_RMC    0x20
#define NMEA_MESSAGE_BIT_VTG    0x40
#define NMEA_MESSAGE_BIT_GST    0x80

#define NMEA_PARITY_NONE 0
#define NMEA_PARITY_ODD  1
#define NMEA_PARITY_EVEN 2

typedef struct {
	INT32 DstEncoded;				// ==0, time code never offset by daylight time
									// ==1, time code offset by daylight time during daylight period
	INT32 FreewheelDisplay;			// ==0, time code does not contain Masterclock, Inc proprietary freewheeling indicator in SMPTE frame
									// ==1, when freewheeling, time code will indicate such using Masterclock, Inc proprietary lock indicator in SMPTE frame
	INT32 FreewheelSynchronize;		// ==0, when freewheeling (FreewheelDisplay must ==1 to know when freewheeling), do not set system time
									// ==1, when freewheeling (FreewheelDisplay must ==1 to know when freewheeling), set system time to incoming time code
	INT32 TimeCodeLossNotify;		// ==0, do not generate alarm when time code lock is lost
									// ==1, generate alarm when time code lock is lost
	INT32 PciComPort;        		// serial port number to use
	INT32 PciNmeaVirtualComPort;   		// secondary serial port on the card for NMEA message output

	// settings for outputting the NMEA messages
	INT32 iNmeaMessagesBitMap;		// See NMEA_MESSAGE_xxx above
	INT32 iNmeaBaudrate;			// 4800, 9600, 19200, 38400, 57600, 115200
	INT32 iNmeaDataBits;            // 7, 8
	INT32 iNmeaParity;				// 0, 1, 2 (see NMEA_PARITY_xxx above)
	INT32 iNmeaStopBits;            // 1,2
	INT32 iNmeaComPortNumber;

	INT32 TimeCodeSourceSmpte;		// see TC_SOURCE_DATE_NOT_INCLUDED, etc.
	INT32 TimeCodeSourceIrig;		// see TC_SOURCE_DATE_NOT_INCLUDED, etc.
	INT32 TimeZoneBiasSeconds;		// time zone bias, in seconds
	INT32 StandardBiasSeconds;		// standard bias, in seconds (usually zero)
	INT32 StandardYear;				// zero for relative standard time, else year for absolute standard date (usually zero)
	INT32 StandardMonth;			// month standard time begins
	INT32 StandardDayOfWeek;		// day of week standard time begins, SUN==1, MON==2, TUE==3, etc.
	INT32 StandardDate;             // date of month standard time begins
	INT32 StandardHour;				// hour of day standard time begins
	INT32 StandardMinute;           // minute of hour standard time begins
	INT32 DaylightBiasSeconds;		// daylight bias, in seconds (usually zero)
	INT32 DaylightYear;				// zero for relative daylight time, else year for absolute daylight date (usually zero)
	INT32 DaylightMonth;			// month daylight time begins
	INT32 DaylightDayOfWeek;		// day of week daylight time begins, SUN==1, MON==2, TUE==3, etc.
	INT32 DaylightDate;				// date of month daylight time begins
	INT32 DaylightHour;				// hour of day daylight time begins
	INT32 DaylightMinute;			// minute of hour daylight time begins
} MCR_CONFIGURATION;

//
// a structure defining a SMPTE time code frame
//



// Communication statuses between MCR device driver and MCR
// note: These defines must match the defiens in McrSerialProtocol.h: HW_STATUS_NORMAL, HW_STATUS_CARDCOMMFAIL 
#define API_HW_STATUS_NORMAL        0x00
#define API_HW_STATUS_FAILED        0x01

// time code types

#define TC_TYPE_SMPTE			0x00000001L
#define TC_TYPE_IRIG			0x00000002L	// deprecated
#define TC_TYPE_IRIG_B1			0x00000002L	// originally, IRIG was only B1
#define TC_TYPE_IRIG_B			0x00000003L
#define TC_TYPE_UNDEFINED		0x00008000L

// time code sub-types
// Note: not sure if these get returned from the driver
#define TC_SUBTYPE_MASK			0x00FF0000L
#define TC_SUBTYPE_SMPTE30FPS	0x00010000L
#define TC_SUBTYPE_EBU25FPS		0x00020000L
#define TC_SUBTYPE_FILM24FPS	0x00040000L
#define TC_SUBTYPE_IRIG_B1		0x00080000L
#define TC_SUBTYPE_IRIG_B		0x00100000L
#define TC_SUBTYPE_UNDEFINED	0x80000000L


// read time code statuses (mapped to kernel-driver statuses)

#define DLL_TC_STATUS_NORMAL	0x00	// time code read was successful
#define DLL_TC_STATUS_INVALID	0x01	// time code data is invalid
#define DLL_TC_STATUS_LOST		0x02	// time code not available
#define DLL_TC_STATUS_DRVFAIL	0x03	// MCR driver communication failure
#define DLL_TC_STATUS_CALIB		0x04	// time code signal calibration in progress

// GPS-200A/MCR1000/MCR5000 operation statuses when Masterclock, Inc proprietary control
// bits are included in SMPTE time code, Leitch date encoding

#define GPS_STATUS_FREEWHEEL	0x00000001

// DLL intialization statuses

#define DLL_INIT_OK				0		// initialization was successful
#define DLL_INIT_CFGERR			3		// registry configuration is invalid or missing
#define DLL_INIT_NOKMD			4		// unable to find the driver, which is required
#define DLL_INIT_TCPROBLEM		5		// unable to detect time code type
#define DLL_INIT_INV_DLL		6		// problem loading DLL or its functions
#define DLL_INIT_UNEXPECTED		255		// unexpected failure (contact technical support)

// asynchronous notifications (maps to MCR interrupts)

#define ASYNC_OKAY				0x00000000		//
#define ASYNC_TIMESET			0x00000001		// once/minute interrupt (sec=30, frame=0)
#define ASYNC_TIMEGAIN			0x00000002		// time code regained from lost state interrupt
#define ASYNC_TIMELOSS			0x00000004		// time code lost interrupt
#define ASYNC_ENDTC				0x00000008		// once/frame interrupt (after time code, before sync word)
#define ASYNC_STARTTC			0x00000010		// once/frame interrupt (before time code, after sync word)
#define ASYNC_PERSEC			0x00000020		// once/second interrupt (frame=0)
#define PCI_COMMUNICATIONS_LOST 0x00000040      // No PCI communications
#define PCI_COMMUNICATIONS_OKAY 0x00000080      // PCI communications okay
#define ASYNC_ALL				0x7fffffff
#define ASYNC_MASK_ERROR		0x80000000




typedef struct _MCR_TIME_REV_A {

	// What type of timecode are we decoding

	DWORD			dwTimeCodeType;          // only gets set on the MCR_ApiPollTimeCode() API call, otherwise zero filled

	// raw time code values
	uTimeCode		rawTc;                   // only gets set on the MCR_ApiPollTimeCode() API call, otherwise zero filled

	// Contains the value of the high performance counter at either top of second
	// or when the MCR_ApiPollTimeCode() command was executed
	LARGE_INTEGER	intPerfCount;            

	// decoded time

	WORD			wHour;
	WORD			wMinute;
	WORD			wSecond;
	WORD			wMilliseconds;			// If intPerfCount is used to calculate a msec value do not include
											// wMilliseconds in the calculation. wMilliseconds is an approximated
											// value, replace it using intPerfCount for a more exact milleseconds value.
	WORD			wMonth;
	WORD			wDay;
	WORD			wYear;

	// validity flag (does checksum pass?  TCR-500/500PCI only)

	BYTE			tcrValid;

	//DWORD          dwLockStatus;            // added for PCIe/PCI66, system lock status, GPS/time code/HSO/RTC

} MCR_TIME_REV_A, *PMCR_TIME_REV_A;


// note: These defines must match defines in McrSerialProtocol.h LOCK_INVALID, LOCK_NOT_PRESENT, LOCK_LOST, LOCK_RESTORED, LOCK_PRESENT
#define API_LOCK_INVALID         0x00   // never had a lock
#define API_LOCK_NOT_PRESENT     0x01   // had a lock but lost it
#define API_LOCK_LOST            0x02   // notify, one time, that lock was lost
#define API_LOCK_RESTORED        0x03   // notify, one time, that lock was restored
#define API_LOCK_PRESENT         0x04   // currently locked to a reference

typedef struct _API_LAST_LOCK_STATUS
{
	BYTE ucLockStatus;
	BYTE ucRtcStatus;
	BYTE ucGpsStatus;
	BYTE ucTimeCodeStatus;
	BYTE ucHsoStatus;
	BYTE ucHsoAchievementLevel;
	BYTE ucReserved1;
	BYTE ucReserved2;
	BYTE ucReserved3;
	BYTE ucReserved4;
} API_LAST_LOCK_STATUS;


typedef DWORD (__stdcall *PMCR_ASYNC_CALLBACK_ROUTINE)(DWORD dwStatus, PVOID data, DWORD dataLen);

#pragma pack(pop)
